@@ -1,48 +1,83 @@
+import logging
 from opcua import Server, ua
 from mtppy.communication_object import OPCUACommunicationObject
 from mtppy.service import Service
-from mtppy.suc_data_assembly import SUCActiveElement
+from mtppy.suc_data_assembly import SUCDataAssembly, SUCActiveElement
+from mtppy.mtp_generator import MTPGenerator
 
 
 class OPCUAServerPEA:
-    def __init__(self, mtp_generator=None, endpoint='opc.tcp://0.0.0.0:4840/'):
+    def __init__(self, mtp_generator: MTPGenerator = None, endpoint: str ='opc.tcp://0.0.0.0:4840/'):
+        """
+        Defines an OPC UA server for PEA.
+        :param mtp_generator: Instance of an MTP generator. If specified, an MTP file is generated each time
+        the class is instantiated. If not specified, no MTP file is going to be generated.
+        :param endpoint: Endpoint of the OPC UA server. If not specified, opc.tcp://0.0.0.0:4840/ is used.
+        """
         self.service_set = {}
         self.active_elements = {}
         self.endpoint = endpoint
         self.opcua_server = None
         self.opcua_ns = 3
         self.subscription_list = SubscriptionList()
-        self.init_opcua_server()
+        self._init_opcua_server()
         self.mtp = mtp_generator
 
     def add_service(self, service: Service):
+        """
+        Add a service to the PEA.
+        :param service: Service instance.
+        :return:
+        """
         self.service_set[service.tag_name] = service
 
     def add_active_element(self, active_element: SUCActiveElement):
+        """
+        Add an active element to the PEA.
+        :param active_element: Active element (e.g. AnaVlv, BinVlv, etc.)
+        :return:
+        """
         self.active_elements[active_element.tag_name] = active_element
 
-    def init_opcua_server(self):
+    def _init_opcua_server(self):
+        """
+        Initialises an OPC UA server and sets the endpoint.
+        :return:
+        """
+        logging.info(f'Initialisation of OPC UA server: {self.endpoint}')
         self.opcua_server = Server()
         self.opcua_server.set_endpoint(self.endpoint)
         # self.opcua_ns = self.opcua_server.register_namespace('namespace_idx')
 
     def get_opcua_server(self):
+        """
+        Get an OPC UA server instance object.
+        :return:
+        """
         return self.opcua_server
 
     def get_opcua_ns(self):
+        """
+        Get an OPC UA server namespace index.
+        :return:
+        """
         return self.opcua_ns
 
     def run_opcua_server(self):
+        """
+        Starts the OPC UA server instance.
+        :return:
+        """
         self.opcua_server.start()
-        self.build_opcua_server()
-        self.start_subscription()
-        # self.set_services_in_idle()
+        self._build_opcua_server()
+        self._start_subscription()
 
-    def set_services_in_idle(self):
-        for service in self.service_set.values():
-            service.init_idle_state()
-
-    def build_opcua_server(self):
+    def _build_opcua_server(self):
+        """
+        Creates an OPC UA server instance including required nodes according to defined data assemblies.
+        :return:
+        """
+        logging.info(f'Adding OPC UA nodes to the server structure according to the PEA structure:')
         ns = self.opcua_ns
         server = self.opcua_server.get_objects_node()
 
@@ -58,11 +93,13 @@ class OPCUAServerPEA:
             self.mtp.add_opcua_server(self.endpoint)
 
         for service in self.service_set.values():
+            logging.info(f'- service {service.tag_name}')
             self._create_opcua_objects_for_folders(service, services_node_id, services_node)
 
         act_elem_node_id = f'ns={ns};s=active_elements'
         act_elem_node = server.add_folder(act_elem_node_id, "active_elements")
         for active_element in self.active_elements.values():
+            logging.info(f'- active element {active_element.tag_name}')
             self._create_opcua_objects_for_folders(active_element, act_elem_node_id, act_elem_node)
 
         # add SupportedRoleClass to all InternalElements
@@ -71,18 +108,25 @@ class OPCUAServerPEA:
 
         # export manifest.aml
         if self.mtp:
+            logging.info(f'MTP manifest export to {self.mtp.export_path}')
             self.mtp.export_manifest()
 
-    def _create_opcua_objects_for_folders(self, data_assembly, parent_opcua_prefix, parent_opcua_object):
+    def _create_opcua_objects_for_folders(self, data_assembly: SUCDataAssembly, parent_opcua_prefix: str, parent_opcua_object):
+        """
+        Iterates over data assemblies to create OPC UA folders.
+        :param data_assembly: Data assembly.
+        :param parent_opcua_prefix: Prefix as a string to add in front of the data assembly tag.
+        :param parent_opcua_object: Parent OPC UA node where the data assembly is to add to.
+        :return:
+        """
         da_node_id = f'{parent_opcua_prefix}.{data_assembly.tag_name}'
         da_node = parent_opcua_object.add_folder(da_node_id, data_assembly.tag_name)
 
         # type of data assembly (e.g. services, active_elements, procedures etc.)
         da_type = parent_opcua_prefix.split('=')[-1].split('.')[-1]
 
-        folders = ['configuration_parameters',
-                   'procedures',
-                   'procedure_parameters', 'process_value_ins', 'report_values', 'process_value_outs']
+        folders = ['configuration_parameters', 'procedures','procedure_parameters',
+                   'process_value_ins', 'report_values', 'process_value_outs']
         leaves = ['op_src_mode', 'state_machine', 'procedure_control']
 
         # create instance of  ServiceControl, HealthStateView, DIntServParam etc.
@@ -106,7 +150,6 @@ class OPCUAServerPEA:
                 continue
             section = eval(f'data_assembly.{section_name}')
             section_node_id = f'{da_node_id}.{section_name}'
-            print(section_node_id)
             section_node = da_node.add_folder(section_node_id, section_name)
             if section_name in folders:
                 for parameter in eval(f'data_assembly.{section_name}.values()'):
@@ -118,15 +161,23 @@ class OPCUAServerPEA:
         if self.mtp:
             self.mtp.add_linked_attr(instance, link_id)
 
-    def _create_opcua_objects_for_leaves(self, object, parent_opcua_prefix, parent_opcua_object, par_instance):
-        for attr in object.attributes.values():
+    def _create_opcua_objects_for_leaves(self, opcua_object, parent_opcua_prefix: str, parent_opcua_object, par_instance):
+        """
+        Iterates over end objects (leaves) of data assemblies to create corresponding OPC UA nodes.
+        :param opcua_object: Element of a data assembly that an OPC UA node is to create for.
+        :param parent_opcua_prefix: Prefix as a string to add in front of the data assembly tag.
+        :param parent_opcua_object: Parent OPC UA node where the data assembly is to add to.
+        :param par_instance: Parameter instance.
+        :return:
+        """
+        for attr in opcua_object.attributes.values():
             attribute_node_id = f'{parent_opcua_prefix}.{attr.name}'
 
             # We attach communication objects to be able to write values on opcua server on attributes change
-            opcua_type = self.infere_data_type(attr.type)
+            opcua_type = self._infer_data_type(attr.type)
             opcua_node_obj = parent_opcua_object.add_variable(attribute_node_id, attr.name, attr.init_value,
                                                               varianttype=opcua_type)
-            print(f'OPCUA Node: {attribute_node_id}, Name: {attr.name}, Value: {attr.init_value}')
+            logging.debug(f'OPCUA Node: {attribute_node_id}, Name: {attr.name}, Value: {attr.init_value}')
             opcua_node_obj.set_writable(False)
             opcua_comm_obj = OPCUACommunicationObject(opcua_node_obj, node_id=opcua_node_obj)
             attr.attach_communication_object(opcua_comm_obj)
@@ -146,7 +197,7 @@ class OPCUAServerPEA:
             added to InstanceHierarchy_Service/service/procedure. The other attributes of procedure should belong to 
             InstanceList/HeathStateView
             """
-            if type(object).__name__ == 'Procedure' and attr.name in ['ProcedureId', 'IsSelfCompleting', 'IsDefault']:
+            if type(opcua_object).__name__ == 'Procedure' and attr.name in ['ProcedureId', 'IsSelfCompleting', 'IsDefault']:
                 pass
             else:
                 if self.mtp:
@@ -157,7 +208,13 @@ class OPCUAServerPEA:
                 opcua_node_obj.set_writable(True)
                 self.subscription_list.append(opcua_node_obj, attr.sub_cb)
 
-    def infere_data_type(self, attribute_data_type):
+    @staticmethod
+    def _infer_data_type(attribute_data_type):
+        """
+        Translate a python data type to a suitable OPC UA data type.
+        :param attribute_data_type: Python variable.
+        :return: OPC UA data type
+        """
         if attribute_data_type == int:
             return ua.VariantType.Int64
         elif attribute_data_type == float:
@@ -169,7 +226,11 @@ class OPCUAServerPEA:
         else:
             return None
 
-    def start_subscription(self):
+    def _start_subscription(self):
+        """
+        Subscribes to defined OPC UA nodes.
+        :return:
+        """
         handler = Marshalling()
         handler.import_subscription_list(self.subscription_list)
         sub = self.opcua_server.create_subscription(500, handler)
@@ -178,13 +239,26 @@ class OPCUAServerPEA:
 
 class SubscriptionList:
     def __init__(self):
+        """
+        Subscription list that contains all OPC UA nodes that PEA must subscribe to.
+        """
         self.sub_list = {}
 
     def append(self, node_id, cb_value_change):
+        """
+        Add an subscription entity.
+        :param node_id: OPC UA node.
+        :param cb_value_change: Callback function for a value change.
+        :return:
+        """
         identifier = node_id.nodeid.Identifier
         self.sub_list[identifier] = {'node_id': node_id, 'callback': cb_value_change}
 
     def get_nodeid_list(self):
+        """
+        Extract a list of node ids in the subscription list.
+        :return: List of node ids.
+        """
         if len(self.sub_list) == 0:
             return None
         else:
@@ -194,6 +268,11 @@ class SubscriptionList:
             return node_id_list
 
     def get_callback(self, node_id):
+        """
+        Get a callback function for a specific OPC UA node.
+        :param node_id: OPC UA node id
+        :return:
+        """
         identifier = node_id.nodeid.Identifier
         if identifier in self.sub_list.keys():
             return self.sub_list[identifier]['callback']
@@ -203,19 +282,38 @@ class SubscriptionList:
 
 class Marshalling(object):
     def __init__(self):
+        """
+        Supplementary class for marshalling OPC UA subscriptions.
+        """
         self.subscription_list = None
 
     def import_subscription_list(self, subscription_list: SubscriptionList):
+        """
+        Import a subscription list.
+        :param subscription_list: Subscription list.
+        :return:
+        """
         self.subscription_list = subscription_list
 
-    def datachange_notification(self, node, val, data):
-        # print("Data change event", node, val)
+    def data_change_notification(self, node, val, data):
+        """
+        Executes a callback function if data value changes.
+        :param node: OPC UA node.
+        :param val: Value after change.
+        :param data: Not used.
+        :return:
+        """
         callback = self.find_set_callback(node)
         if callback is not None:
             try:
                 callback(val)
             except Exception as exc:
-                print(f'Something wrong with callback {callback}: {exc}')
+                logging.warning(f'Something wrong with callback {callback}: {exc}')
 
     def find_set_callback(self, node_id):
+        """
+        Finds a callback function to a specific OPC UA node by nodeid.
+        :param node_id: Node id.
+        :return: Callback function.
+        """
         return self.subscription_list.get_callback(node_id)
