@@ -1,9 +1,16 @@
+import logging
+
 from mtppy.attribute import Attribute
 from mtppy.operation_source_mode import OperationSourceMode
 
 
 class ProcedureControl:
     def __init__(self, procedures: dict, service_op_src_mode: OperationSourceMode):
+        """
+        Represents the procedure control.
+        :param procedures: Procedures.
+        :param service_op_src_mode: Operation and source mode of the service.
+        """
         self.attributes = {
             'ProcedureOp': Attribute('ProcedureOp', int, init_value=0, sub_cb=self.set_procedure_op),
             'ProcedureInt': Attribute('ProcedureInt', int, init_value=0, sub_cb=self.set_procedure_int),
@@ -17,17 +24,17 @@ class ProcedureControl:
         self.default_procedure_id = None
 
     def set_procedure_op(self, value: int):
-        print('ProcedureOP set to %s' % value)
+        logging.debug('ProcedureOP set to %s' % value)
         if self.op_src_mode.attributes['StateOpAct'].value:
             self.set_procedure_req(value)
 
     def set_procedure_int(self, value: int):
-        print('ProcedureInt set to %s' % value)
+        logging.debug('ProcedureInt set to %s' % value)
         if self.op_src_mode.attributes['StateAutAct'].value and self.op_src_mode.attributes['SrcIntAct'].value:
             self.set_procedure_req(value)
 
     def set_procedure_ext(self, value: int):
-        print('ProcedureExt set to %s' % value)
+        logging.debug('ProcedureExt set to %s' % value)
         if self.op_src_mode.attributes['StateAutAct'].value and self.op_src_mode.attributes['SrcExtAct'].value:
             self.set_procedure_req(value)
 
@@ -40,14 +47,14 @@ class ProcedureControl:
     def set_procedure_req(self, value: int):
         if self.valid_value(value):
             self.attributes['ProcedureReq'].set_value(value)
-            print('ProcedureReq set to %s' % value)
+            logging.debug('ProcedureReq set to %s' % value)
         else:
-            print('ProcedureReq cannot be set to %s (out of range)' % value)
+            logging.debug('ProcedureReq cannot be set to %s (out of range)' % value)
 
     def set_procedure_cur(self):
         procedure_req = self.attributes['ProcedureReq'].value
         self.attributes['ProcedureCur'].set_value(procedure_req)
-        print('ProcedureCur set to %s' % procedure_req)
+        logging.debug('ProcedureCur set to %s' % procedure_req)
 
     def get_procedure_cur(self):
         return self.attributes['ProcedureCur'].value
